@@ -166,7 +166,12 @@ function setupPageDom (pageView) {
     }
     textLayer = pageView.textLayerFactory.createTextLayerBuilder(textLayerDiv, pageView.id - 1, pageView.viewport, pageView.enhanceTextSelection)
   }
+  if (pageView.annotationLayerFactory) {
+    var annotationLayer = pageView.annotationLayerFactory.createAnnotationLayerBuilder(div, pdfPage, null, null, false, pageView.l10n, null, null, null, null, null)
+  }
   pageView.textLayer = textLayer
+  pageView.annotationLayer = annotationLayer
+  setUpPageAnnotationLayer(pageView)
 }
 
 function DefaultTextLayerFactory () { }
@@ -214,6 +219,29 @@ const updateCachedPages = throttle(function () {
 
 var pageCount
 
+function setUpPageAnnotationLayer (pageView) {
+  pageView.annotationLayer.linkService.goToDestination = async function (dest) {
+    // Adapted from https://github.com/mozilla/pdf.js/blob/8ac0ccc2277a7c0c85d6fec41c0f3fc3d1a2d232/web/pdf_link_service.js#L238
+    let explicitDest
+    if (typeof dest === 'string') {
+      explicitDest = await pdf.getDestination(dest)
+    } else {
+      explicitDest = await dest
+    }
+
+    const destRef = explicitDest[0]
+    let pageNumber
+
+    if (typeof destRef === 'object' && destRef !== null) {
+      pageNumber = await pdf.getPageIndex(destRef)
+    } else if (Number.isInteger(destRef)) {
+      pageNumber = destRef + 1
+    }
+
+    pageViews[pageNumber].div.scrollIntoView()
+  }
+}
+
 pdfjsLib.getDocument({ url: url, withCredentials: true }).promise.then(async function (pdf) {
   window.pdf = pdf
 
@@ -275,23 +303,11 @@ pdfjsLib.getDocument({ url: url, withCredentials: true }).promise.then(async fun
         updateGutterWidths()
       }
 
-      function setupPageNavigation (pageView) {
-        pageView.annotationLayer.linkService.navigateTo = function (loc) {
-          // TODO support more types of links
-          if (loc.startsWith('p')) {
-            var pageNumber = parseInt(loc.replace('p', ''))
-            pageViews[pageNumber].div.scrollIntoView()
-          } else {
-            console.warn('unsupported link : ' + loc)
-          }
-        }
-      }
-
       (function (pageNumber, pdfPageView) {
         setTimeout(function () {
           if (pageNumber < pageBuffer || (currentPage && Math.abs(currentPage - pageNumber) < pageBuffer)) {
             pageContainer.classList.add('loading')
-            pdfPageView.draw().then(function () { setupPageNavigation(pdfPageView) }).then(function () {
+            pdfPageView.draw().then(function () { setUpPageAnnotationLayer(pdfPageView) }).then(function () {
               pageContainer.classList.remove('loading')
               if (pageNumber === 1) {
                 showViewerUI()
@@ -309,6 +325,7 @@ pdfjsLib.getDocument({ url: url, withCredentials: true }).promise.then(async fun
               pdfPageView.pdfPage.getTextContent({ normalizeWhitespace: true }).then(function (text) {
                 pdfPageView.textLayer.setTextContent(text)
                 pdfPageView.textLayer.render(0)
+                pdfPageView.annotationLayer.render(pdfPageView.viewport, 'display')
               })
             }, { timeout: 10000 })
           }
